@@ -1,6 +1,6 @@
 /* ::[[ @(#) flip.c 1.18 89/07/04 16:07:16 ]]:: */
 #ifndef LINT
-static char sccsid[]="::[[ @(#) flip.c 1.18 89/07/04 16:07:16 ]]::";
+static char sccsid[]="::[[ @(#) flip.c 1.20 2008-07-26 ]]::";
 #endif
 
 /*
@@ -79,7 +79,7 @@ int bintoo = 0;            /* set by -b option */
 int ztrunc = 0;            /* set by -z option */
 int use_stdio = 0;	   /* set by "-" filename */
 
-main (argc, argv)
+int main (argc, argv)
 int argc;
 char **argv;
 
@@ -236,7 +236,7 @@ enum choices which;
    int errstat = 0;
    char *p;             /* temp file ptr */
 #ifdef IX
-   struct stat ifilestat;
+   struct stat ifilestat, ofilestat;
 #endif
 
 #ifdef USE_SIG
@@ -260,6 +260,8 @@ enum choices which;
 	     perror(buf);
 	   }
 	   ifilestat.st_mode = 0644;
+	   ifilestat.st_uid = geteuid();
+	   ifilestat.st_gid = getegid();
 	 }
 #endif
       }
@@ -332,6 +334,8 @@ enum choices which;
     case MSTOIX:
       errstat = mstoix (infile, outfile);
       break;
+    case NEITHER:
+      ;
    }
 
    fclose (infile);
@@ -358,7 +362,36 @@ enum choices which;
 
    if (!ferror(outfile) && fflush(outfile) != EOF && fclose(outfile) != EOF) {
       int moved;
-      DELFILE (fname);
+#ifdef IX
+      if (stat (tfname, &ofilestat)) {
+	/* can't get the file's permissions */
+	char *buf = malloc(strlen(tfname)+strlen(myname)+3);
+	if (buf) {
+	  sprintf(buf, "%s: %s", myname, tfname);
+	  perror(buf);
+	}
+      }
+      if (ifilestat.st_gid != ofilestat.st_gid ||
+	  ifilestat.st_uid != ofilestat.st_uid)
+	if (chown(tfname, ifilestat.st_uid, ifilestat.st_gid)) {
+	  /* can't set the file's ownership */
+	  char *buf = malloc(strlen(tfname)+strlen(myname)+3);
+	  if (buf) {
+	    sprintf(buf, "%s: %s", myname, tfname);
+	    perror(buf);
+	  }
+	}
+      if (ifilestat.st_mode != ofilestat.st_mode)
+	if (chmod(tfname, ifilestat.st_mode)) {
+	  /* can't set the file's permissions */
+	  char *buf = malloc(strlen(tfname)+strlen(myname)+3);
+	  if (buf) {
+	    sprintf(buf, "%s: %s", myname, tfname);
+	    perror(buf);
+	  }
+	}
+#endif /* IX */
+      //      DELFILE (fname);
       if (!use_stdio) {
          moved = MVFILE (tfname, fname);
          if (moved == 0) {
@@ -367,16 +400,6 @@ enum choices which;
                SETFT (fptr, fname, timestamp);
                fclose (fptr);
             }
-#ifdef IX
-	    if (chmod(fname, ifilestat.st_mode)) {
-	      /* can't set the file's permissions */
-	      char *buf = malloc(strlen(fname)+strlen(myname)+3);
-	      if (buf) {
-		sprintf(buf, "%s: %s", myname, fname);
-		perror(myname);
-	      }
-	    }
-#endif
             if (verbose) {
                printf ("%s\n", fname);
                fflush (stdout);
@@ -392,6 +415,7 @@ enum choices which;
       fclose (outfile); /* outfile was not closed, so close it here */
       return (1);
    }
+   return (1);			/* silences gcc warning */
 }
 
 /* convert from ms-dos to **ix format */
@@ -433,7 +457,7 @@ FILE *outfile;          /* output file  */
  saweof:
       /* exit outer loop only on EOF or ^Z as last char */
       if (
-          ztrunc && state == SAWCTRLZ
+          (ztrunc && (state == SAWCTRLZ))
           || (c = getc (infile)) == EOF
          )
          break;
